"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateApiClient = generateApiClient;
const utils_1 = require("../utils");
function resolveSchema(schema, spec) {
    if (!schema)
        return undefined;
    if ("$ref" in schema) {
        const index = schema.$ref.split("/").pop();
        return spec.components?.schemas?.[index];
    }
    return schema;
}
function generateAxiosMethod(operation, spec) {
    const { method, path, operationId, summary, description, parameters, requestBody, responses } = operation;
    // Generate JSDoc
    const jsDocLines = ["/**"];
    if (summary)
        jsDocLines.push(` * ${summary}`);
    if (description)
        jsDocLines.push(` * ${description}`);
    // Add parameter descriptions
    parameters?.forEach((param) => {
        const desc = param.description ? ` - ${param.description}` : "";
        jsDocLines.push(` * @param ${param.in === "path" ? "params." : param.in === "query" ? "query." : ""}${param.name}${desc}`);
    });
    if (requestBody && "description" in requestBody) {
        jsDocLines.push(` * @param data - ${requestBody.description}`);
    }
    // Add return type description
    const responseDetails = Object.entries(responses).find(([code]) => code.startsWith("2"));
    if (responseDetails) {
        const [code, response] = responseDetails;
        const responseObj = response;
        const desc = "description" in responseObj ? responseObj.description : "";
        const contentType = responseObj.content?.["application/ld+json"]?.schema ??
            responseObj.content?.["application/json"]?.schema ??
            responseObj.content?.["application/octet-stream"]?.schema;
        const typeName = (0, utils_1.pascalCase)(`${operationId}Response${code}`);
        if (contentType) {
            if (desc) {
                jsDocLines.push(` * @returns ${desc}`);
            }
            jsDocLines.push(` * @see ${typeName}`);
        }
        else if (desc) {
            jsDocLines.push(` * @returns ${desc}`);
        }
    }
    jsDocLines.push(" */");
    const urlParams = parameters?.filter((p) => p.in === "path") || [];
    const queryParams = parameters?.filter((p) => p.in === "query") || [];
    const isFormData = requestBody && "content" in requestBody && requestBody.content?.["multipart/form-data"];
    const formDataSchema = isFormData
        ? resolveSchema(requestBody.content["multipart/form-data"].schema, spec)
        : undefined;
    const content = requestBody && "content" in requestBody
        ? (requestBody.content?.["application/ld+json"]?.schema ??
            requestBody.content?.["application/json"]?.schema ??
            requestBody.content?.["application/octet-stream"]?.schema)
        : undefined;
    const requestBodySchema = content ? resolveSchema(content, spec) : undefined;
    // Add request body type if it exists
    const hasData = (parameters && parameters.length > 0) || operation.requestBody;
    const namedType = (0, utils_1.pascalCase)(operationId);
    // Get response type from 2xx response
    const responseType = responseDetails?.[0] && "content" in responseDetails[1]
        ? `T.${`${namedType}Response${responseDetails[0]}`}`
        : "unknown";
    const urlWithParams = urlParams.length > 0 ? `\`${path.replace(/{(\w+)}/g, "${data.$1}")}\`` : `"${path}"`;
    const methodBody = [
        `${hasData ? "const { axiosConfig = {}, ...data } = props || {};" : "const { axiosConfig } = props || {};"}`,
        "const apiClient = getApiClient();",
        `const url = ${urlWithParams};`,
        queryParams.length > 0
            ? `const queryData = {
				${queryParams.map((p) => `["${p.name}"]: data["${p.name}"]`).join(",\n				")}
			};`
            : "",
        requestBodySchema?.properties
            ? `const bodyData = {
				${Object.entries(requestBodySchema.properties)
                .map(([key]) => `["${key}"]: data["${key}"]`)
                .join(",\n				")}
			};`
            : "",
        formDataSchema?.properties
            ? `const bodyData = new FormData();
			${Object.entries(formDataSchema.properties)
                .map(([key, prop]) => {
                const schemaProperty = prop;
                const isBinary = schemaProperty.format === "binary";
                return formDataSchema?.required?.includes(key)
                    ? `bodyData.append("${key}", ${isBinary ? "" : "String("}${queryParams.length > 0 ? "bodyData" : "data"}.${key}${isBinary ? "" : ")"});`
                    : `if (${queryParams.length > 0 ? "bodyData" : "data"}.${key} != null) {
							bodyData.append("${key}", ${isBinary ? "" : "String("}${queryParams.length > 0 ? "bodyData" : "data"}.${key}${isBinary ? "" : ")"});
						}`;
            })
                .join("\n			")}`
            : "",
        queryParams.length > 0 ? "axiosConfig.params = queryData;" : "",
        isFormData
            ? "axiosConfig.headers = { ...axiosConfig.headers, 'Content-Type': 'multipart/form-data' };"
            : "",
        requestBody
            ? `const res = await apiClient.${method}<${responseType}>(url, bodyData, axiosConfig);`
            : `const res = await apiClient.${method}<${responseType}>(url, axiosConfig);`,
        "return res.data;",
    ]
        .filter(Boolean)
        .join("\n	");
    // ${queryParams.length > 0 ? "params: queryData," : ""}
    // ${requestBody ? `data: ${isFormData ? "formData" : "bodyData"},` : ""}
    // ${isFormData ? `config: { headers: { 'Content-Type': 'multipart/form-data', ...axiosConfig?.headers }, ...axiosConfig },` : "...axiosConfig"}
    const requestParms = hasData
        ? `props: T.${(0, utils_1.pascalCase)(operationId)}Params & { axiosConfig?: AxiosRequestConfig; }`
        : "props?: { axiosConfig?: AxiosRequestConfig }";
    return `
	${jsDocLines.join("\n	")}
	export async function ${(0, utils_1.camelCase)(operationId)}(${requestParms}): Promise<${responseType}> {
		${methodBody}
	}`;
}
function generateApiClient(spec) {
    const operations = [];
    const resolveParameters = (parameters) => {
        return parameters.map((p) => {
            if ("$ref" in p) {
                const index = p.$ref.split("/").pop();
                return spec.components?.schemas?.[index];
            }
            return p;
        });
    };
    const resolveRequestBody = (requestBody) => {
        if (!requestBody)
            return undefined;
        if ("$ref" in requestBody) {
            const index = requestBody.$ref.split("/").pop();
            return spec.components?.schemas?.[index];
        }
        return requestBody;
    };
    // Collect all operations
    Object.entries(spec.paths || {}).forEach(([path, pathItem]) => {
        if (!pathItem)
            return;
        ["get", "post", "put", "delete", "patch"].forEach((method) => {
            const operation = pathItem[method];
            if (!operation)
                return;
            operations.push({
                method: method,
                path,
                operationId: `${(0, utils_1.sanitizeTypeName)(operation.operationId || `${path.replace(/\W+/g, "_")}`)}`,
                summary: operation.summary,
                description: operation.description,
                parameters: resolveParameters([...(pathItem.parameters || []), ...(operation.parameters || [])]),
                requestBody: resolveRequestBody(operation.requestBody),
                responses: operation.responses,
            });
        });
    });
    const title = (0, utils_1.specTitle)(spec);
    return `import type { AxiosResponse, AxiosRequestConfig } from 'axios';
import { getApiClient } from './apiClient';
import type * as T from './${title}.schema';

${operations.map((op) => generateAxiosMethod(op, spec)).join("\n\n")}`;
}
