"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateTypeDefinitions = generateTypeDefinitions;
const utils_1 = require("../utils");
function generateTypeDefinition(name, schema) {
    const description = !("$ref" in schema) && schema.description ? `/**\n * ${schema.description}\n */\n` : "";
    const typeValue = (0, utils_1.getTypeFromSchema)(schema);
    // Use 'type' for primitives, unions, and simple types
    // Use 'interface' only for complex objects with properties
    const isInterface = !("$ref" in schema) && schema.type === "object" && schema.properties;
    return isInterface
        ? `${description}export interface ${(0, utils_1.sanitizeTypeName)(name)} ${typeValue}\n\n`
        : `${description}export type ${(0, utils_1.sanitizeTypeName)(name)} = ${typeValue}\n\n`;
}
/**
 * Generates TypeScript interface definitions from OpenAPI schemas
 */
function generateTypeDefinitions(spec) {
    const context = {
        schemas: spec.components?.schemas || {},
        generatedTypes: new Set(),
    };
    let output = "/* Generated TypeScript Definitions */\n\n";
    // Generate types for all schema definitions
    for (const [name, schema] of Object.entries(context.schemas)) {
        if (context.generatedTypes.has(name))
            continue;
        output += generateTypeDefinition(name, schema);
        context.generatedTypes.add(name);
    }
    // Generate request/response types
    if (spec.paths) {
        for (const [path, pathItem] of Object.entries(spec.paths)) {
            for (const [method, operation] of Object.entries(pathItem)) {
                if (method === "$ref")
                    continue;
                const operationObject = operation;
                if (!operationObject)
                    continue;
                const { operationId: badOperationId, requestBody, responses, parameters } = operationObject;
                const operationId = `${(0, utils_1.sanitizeTypeName)(badOperationId || `${path.replace(/\W+/g, "_")}`)}`;
                // Generate request body type
                if (requestBody) {
                    const content = requestBody.content;
                    const jsonContent = content["application/ld+json"] ??
                        content["application/json"] ??
                        content["multipart/form-data"] ??
                        content["application/octet-stream"];
                    if (jsonContent?.schema) {
                        const typeName = `${operationId}Request`;
                        output += generateTypeDefinition(typeName, jsonContent.schema);
                    }
                }
                // Generate response types
                if (responses) {
                    for (const [code, response] of Object.entries(responses)) {
                        const responseObj = response;
                        const content = responseObj.content?.["application/ld+json"] ??
                            responseObj.content?.["application/json"] ??
                            responseObj.content?.["application/octet-stream"];
                        if (content?.schema) {
                            const typeName = `${operationId}Response${code}`;
                            output += generateTypeDefinition(typeName, content.schema);
                        }
                    }
                }
                // Build data type parts
                const dataProps = [];
                const urlParams = (parameters?.filter((p) => "in" in p && p.in === "path") ||
                    []);
                const queryParams = (parameters?.filter((p) => "in" in p && p.in === "query") ||
                    []);
                // Add path and query parameters
                urlParams.forEach((p) => {
                    const safeName = (0, utils_1.sanitizePropertyName)(p.name);
                    const isDeprecated = "deprecated" in p && p.deprecated;
                    const hasDescription = "description" in p && p.description;
                    const desc = hasDescription || isDeprecated
                        ? `/**${hasDescription ? `\n* ${p.description}` : ""}${isDeprecated ? "\n* @deprecated" : ""}
							*/\n`
                        : "";
                    dataProps.push(`${desc}${safeName}: ${(0, utils_1.getTypeFromSchema)(p.schema)}`);
                });
                queryParams.forEach((p) => {
                    const safeName = (0, utils_1.sanitizePropertyName)(p.name);
                    const isDeprecated = "deprecated" in p && p.deprecated;
                    const hasDescription = "description" in p && p.description;
                    const desc = hasDescription || isDeprecated
                        ? `\n/**${hasDescription ? `\n* ${p.description}` : ""}${isDeprecated ? "\n* @deprecated" : ""}
							*/\n`
                        : "";
                    dataProps.push(`${desc}${safeName}${p.required ? "" : "?"}: ${(0, utils_1.getTypeFromSchema)(p.schema)}`);
                });
                // Add request body type if it exists
                const hasData = (parameters && parameters.length > 0) || requestBody;
                let dataType = "undefined";
                const namedType = (0, utils_1.pascalCase)(operationId);
                if (hasData) {
                    if (requestBody && dataProps.length > 0) {
                        dataType = `${namedType}Request & { ${dataProps.join("; ")} }`;
                    }
                    else if (requestBody) {
                        dataType = `${namedType}Request`;
                    }
                    else if (dataProps.length > 0) {
                        dataType = `{ ${dataProps.join("; ")} }`;
                    }
                    else {
                        dataType = "Record<string, never>";
                    }
                    output += `\n\nexport type ${(0, utils_1.pascalCase)(operationId)}Params = ${dataType};\n\n`;
                }
            }
        }
    }
    return output;
}
