"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.camelCase = camelCase;
exports.pascalCase = pascalCase;
exports.sanitizePropertyName = sanitizePropertyName;
exports.sanitizeTypeName = sanitizeTypeName;
exports.specTitle = specTitle;
exports.getTypeFromSchema = getTypeFromSchema;
function camelCase(str) {
    return str
        .replace(/[^a-zA-Z0-9]+(.)/g, (_, chr) => chr.toUpperCase())
        .replace(/^[A-Z]/, (c) => c.toLowerCase());
}
function pascalCase(str) {
    return str
        .replace(/[^a-zA-Z0-9]+(.)/g, (_, chr) => chr.toUpperCase())
        .replace(/^[a-z]/, (c) => c.toUpperCase());
}
/**
 * Sanitizes a property name to ensure it's a valid JavaScript identifier.
 * If the name is already a valid identifier (starts with letter/underscore/$ and contains only letters/numbers/underscore/$),
 * returns it as-is. Otherwise wraps it in quotes to make it a valid property accessor.
 *
 * For example:
 * - sanitizePropertyName("validName") => "validName"
 * - sanitizePropertyName("invalid-name") => "'invalid-name'"
 * - sanitizePropertyName("123invalid") => "'123invalid'"
 *
 * @param name The property name to sanitize
 * @returns The sanitized property name, quoted if needed
 */
function sanitizePropertyName(name) {
    return /^[a-zA-Z_$][a-zA-Z0-9_$]*$/.test(name) ? name : `'${name}'`;
}
/**
 * Sanitizes a type name to ensure it's a valid TypeScript type identifier.
 * Replaces any characters that aren't alphanumeric or underscore with an underscore.
 *
 * For example:
 * - sanitizeTypeName("ValidType") => "ValidType"
 * - sanitizeTypeName("invalid-type") => "invalid_type"
 * - sanitizeTypeName("type.name") => "type_name"
 * - sanitizeTypeName("type/name") => "type_name"
 *
 * This is used to convert operation IDs and response names from the OpenAPI spec
 * into valid TypeScript type names.
 *
 * @param name The type name to sanitize
 * @returns The sanitized type name with invalid characters replaced by underscores
 */
function sanitizeTypeName(name) {
    return pascalCase(name.replace(/[^a-zA-Z0-9_]/g, "_").replace(/_+$/, ""));
}
function specTitle(spec) {
    return camelCase(spec.info.title.toLowerCase().replace(/\s+/g, "-"));
}
/**
 * Converts an OpenAPI schema object into a TypeScript type string.
 *
 * Handles:
 * - References ($ref) by extracting the type name
 * - Nullable types by appending "| null"
 * - Enums by creating union types of the values
 * - Basic types (string, number, boolean)
 * - Binary format strings as a union with file metadata object
 * - Arrays by recursively getting the item type
 * - Objects with properties by creating interfaces
 * - Objects with additionalProperties as Records
 * - Fallback to "any" for unknown types
 *
 * @param param - The OpenAPI schema/parameter object to convert
 * @returns The TypeScript type as a string
 */
function getTypeFromSchema(schema) {
    if (!schema)
        return undefined;
    // Handle $ref by extracting the referenced type name
    if ("$ref" in schema) {
        const refType = schema.$ref.split("/").pop();
        return sanitizeTypeName(refType);
    }
    // Add "| null" for nullable types
    const nullable = "nullable" in schema && schema.nullable ? " | null" : "";
    // Handle enums as union types
    if ("enum" in schema && schema.enum) {
        if (Object.values(schema.enum)?.length > 0) {
            return (Object.values(schema.enum)
                .map((e) => (typeof e === "string" ? `'${e}'` : e))
                .join(" | ") + nullable);
        }
        return schema.enum.map((e) => (typeof e === "string" ? `'${e}'` : e)).join(" | ") + nullable;
    }
    // Handle types based on the "type" property
    if ("type" in schema) {
        switch (schema.type) {
            case "string":
                // Special case for binary format strings
                if ("format" in schema && schema.format === "binary") {
                    return `string | { name?: string; type?: string; uri: string }${nullable}`;
                }
                return `string${nullable}`;
            case "number":
            case "integer":
                return `number${nullable}`;
            case "boolean":
                return `boolean${nullable}`;
            case "array": {
                // Recursively get the array item type
                const itemType = getTypeFromSchema(schema.items);
                return `Array<${itemType}>${nullable}`;
            }
            case "object":
                // Handle objects with defined properties
                if (schema.properties) {
                    const properties = Object.entries(schema.properties)
                        .map(([key, prop]) => {
                        const isRequired = schema.required?.includes(key);
                        const propertyType = getTypeFromSchema(prop);
                        const safeName = sanitizePropertyName(key);
                        const isDeprecated = "deprecated" in prop && prop.deprecated;
                        const hasDescription = "description" in prop && prop.description;
                        const desc = hasDescription || isDeprecated
                            ? `/**${hasDescription ? `\n* ${prop.description}` : ""}${isDeprecated ? "\n* @deprecated" : ""}
									*/\n`
                            : "";
                        return `${desc}${safeName}${isRequired ? "" : "?"}: ${propertyType};`;
                    })
                        .join("\n");
                    return `{${properties}\n}${nullable}`;
                }
                // Handle objects with additionalProperties
                if (schema.additionalProperties) {
                    const valueType = typeof schema.additionalProperties === "boolean"
                        ? "any"
                        : getTypeFromSchema(schema.additionalProperties);
                    return `Record<string, ${valueType}>${nullable}`;
                }
                // Default object type when no properties specified
                return `Record<string, any>${nullable}`;
            default:
                return `any${nullable}`;
        }
    }
    // Fallback for schemas without a type
    return "any";
}
